use actix_web::{http::header::LOCATION, web, HttpResponse, Responder};
use garde::Validate;
use sqlx::PgPool;

use crate::{
    endpoints::events::NewOrEditEventForm,
    models::{Event, EventChangeset, Location, Role, User},
    utils::{self, ApplicationError},
};

#[actix_web::post("/events/new")]
pub async fn post(
    user: web::ReqData<User>,
    pool: web::Data<PgPool>,
    form: web::Form<NewOrEditEventForm>,
) -> Result<impl Responder, ApplicationError> {
    if user.role != Role::Admin && user.role != Role::AreaManager {
        return Err(ApplicationError::Unauthorized);
    }

    let Some(location) = Location::read_by_id(pool.get_ref(), form.location).await? else {
        return Ok(HttpResponse::BadRequest().body("Location does not exist"));
    };

    if user.role != Role::Admin && user.area_id != location.area_id {
        return Ok(HttpResponse::BadRequest().body("Can't use location outside of your area"));
    }

    let changeset = EventChangeset {
        amount_of_posten: form.amount,
        clothing: form.clothing,
        location_id: form.location,
        time: (form.date.and_time(form.start), form.end),
        name: form.name.clone(),
        note: form
            .note
            .clone()
            .and_then(|n| if !n.is_empty() { Some(n) } else { None }),
        voluntary_wachhabender: form.voluntarywachhabender,
        voluntary_fuehrungsassistent: form.voluntaryfuehrungsassistent,
    };

    if let Err(e) = changeset.validate_with(&None) {
        return Ok(HttpResponse::BadRequest().body(e.to_string()));
    };

    Event::create(pool.get_ref(), changeset).await?;

    let url = utils::get_return_url_for_date(&form.date);
    //println!("redirecto to {url}");
    Ok(HttpResponse::Found()
        .insert_header((LOCATION, url.clone()))
        .insert_header(("HX-LOCATION", url))
        .finish())
}
